// CommQueue.h : header file
//
// Copyright(C) 2009 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// THIS SOURCE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//
// IN NO EVENT SHALL UBISYS TECHNOLOGIES BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOURCE CODE OR THE USE OR
// OTHER DEALINGS IN THE SOURCE CODE.
//
// Part of the ubisys(R) 13.56 MHz RFID USB stick demo application
//
// Provides CCommQueue, a CCommunicationsQueue derived class that provides
// multi-threaded data exchange via serial ports or USB CDC/ACM devices,
// such as the 13.56 MHz RFID stick 


/////////////////////////////////////////////////////////////////////////////
// CCommQueue

class CCommQueue : public CCommunicationsQueue
{
	// Construction
	public:
		CCommQueue();

	// Attributes
	public:
		// Specifies the device path to open
		std::string m_strDevice;

	// Operations
	public:
		// Opens the serial port, stores the handle and
		// returns true upon success, false otherwise
		bool Open(const char *const pszPort, 
			const int nDesiredAccess = O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK | O_SYNC);

		// Setup of the serial port configuration (baud rate, etc.)
		bool Setup(const speed_t nBaudRate = B115200, 
			const tcflag_t cflagAdd = CLOCAL | CREAD | CS8, 
			const tcflag_t cflagRemove = PARENB | CSTOPB | CSIZE | CRTSCTS,
			const tcflag_t iflagAdd = 0, const tcflag_t iflagRemove = IXON | IXOFF | IXANY,
			const tcflag_t oflagAdd = 0, const tcflag_t oflagRemove = OPOST);

		// Closes the handle
		bool Close();

		/*
		// Enumerates all plug-and-play serial ports available on the system.
		// Returns an array of serial ports with device path, friendly name,
		// description and a list of associated hardware IDs
		static bool Enumerate(std::vector<CCommPortInfo> &ports, 
			const bool bIgnoreBusy = true);

		// Given an array of available serial port devices, this method 
		// rejects all devices that are not USB devices. If wVendorID is 
		// non-zero, only devices that match the given VID/PID combination
		// are kept
		static bool SelectUSBDevices(std::vector<CCommPortInfo> &ports, 
			const WORD wProductID, const WORD wVendorID = 0x19a6);

		static void RejectBusyDevices(std::vector<CCommPortInfo> &ports);

		// Given a fully qualified device path, this method returns VID/PID
		// and serial number. Notice that not all devices have a serial 
		// number and have arbitrary length
		static bool GetUSBDeviceInformation(const CString &strDevicePath, 
			WORD &wVendorID, WORD &wProductID, CString &strSerialNumber);
		*/

	// Overrides
	public:
		// Clears the RX buffer
		virtual bool PurgeReceptionBuffer();

		// Clears the TX buffer
		virtual bool PurgeTransmissionBuffer();

	// Overridables
	public:
		// Called after data has been successfully written
		virtual void AfterTransmit(const CPacket &packet);

		// Called after data has been successfully received
		virtual void AfterReceive(const CPacket &packet);

	// Implementation
	public:
		virtual ~CCommQueue();

		// The file handle providing access to the comm port device driver
        int m_hFile;

		// The thread function responsible for reading from the comm port
		static void *ReadThread(void *pArgument);

		// The thread function responsible for writing to the comm port
		static void *WriteThread(void *pArgument);

		// Reading thread implementation
		void *ReadThread();

		// Writing thread implementation
		void *WriteThread();
};
