// Packet.h : header file
//
// Copyright(C) 2008 ubisys technologies GmbH, Duesseldorf, Germany.
// All rights reserved.
//
// www.ubisys.de
// support@ubisys.de
//
// Part of the Compact15.4(TM) class library
//
// Adds the ability to dump CPacket objects

#include <cstdlib>
#include "CompactFramework.h"
#include "Packet.h"

/////////////////////////////////////////////////////////////////////////////
// CPacketData

#ifdef _DEBUG

void CPacketData::Dump(const arg_type nStart, const bool bAsHex, 
	const bool bAsText) const
{
	ASSERT(nStart <= static_cast<size_type>(-1));
	
	TRACE6(_T(" [ %p, r = %hu, u = %hu(%hu), a = %hu/%hu ]"), 
		this, m_nReference, GetSize() - nStart, 
		GetSize(), m_nOffset, m_nCapacity - m_nOffset - m_nSize);

	// Hexadecimal representation?	
	if (bAsHex)
	{
		for (size_type i = nStart; i < GetSize(); i++)
			TRACE1(_T(" %02x"), *(static_cast<const unsigned char *>(GetData()) + i));
	}

	// Text (ASCII) representation?
	if (bAsText)
		TRACE2(_T(" - \"%.*hs\""), GetSize(), static_cast<const char *>(GetData()));
	
	TRACE0(_T("\n"));
}

#endif


/////////////////////////////////////////////////////////////////////////////
// CPacket

#ifdef _DEBUG

void CPacket::Dump(const TCHAR *const pszMessage, 
	const CPacketData::arg_type nStart, const bool bAsHex, 
	const bool bAsText) const
{
	if (pszMessage)
		TRACE0(pszMessage);

	if (m_pData)
		m_pData->Dump(nStart, bAsHex, bAsText);
	else
		TRACE0(_T(" <empty>\n"));
}

#endif
